package cn.com.ctop.toutiao.modules.report.mapper;

import com.alibaba.fastjson.JSONObject;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Created by JQ.bi on 2020.05.19
 */
public interface BytedanceHomepageMapper {

    //今日汇总数据
    JSONObject queryTodaySumReportBy(@Param("statDate") String statDate);

    //今日明细数据
    List<JSONObject> queryTodayDetailReportBy(@Param("statDate") String statDate);

    //根据开始、结束时间 查询汇总数据
    JSONObject querySumByStartEndDate(@Param("startDate") String anotherDay, @Param("endDate") String endDate);

    //根据开始、结束时间 查询明细数据
     List<JSONObject> queryDetailByStartEndDate(@Param("startDate") String anotherDay, @Param("endDate") String endDate);

    //按月分组查询明细数据
    List<JSONObject> queryDetailGroupMonthByDate(@Param("startDate") String anotherDay, @Param("endDate") String endDate);

    //查询固定月的明细数据
    List<JSONObject> queryDetailGroupDayByMonth(@Param("month") String month);

    //项目消耗排行榜
    List<JSONObject> queryTopCost(@Param("startDate") String anotherDay, @Param("endDate") String endDate, @Param("rank") int rank);

}
